#include "TArtNEBULAParameters.hh"
#include "TArtNEBULAPlaPara.hh"
#include "TArtCore.h"
#include "TArtStoreManager.h"
#include "TArtMap.h"

#include <TDOMParser.h>
#include <TXMLNode.h>

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <vector>

//__________________________________________________________
TArtNEBULAParameters::TArtNEBULAParameters(const char* name, const char* title)
  : TNamed(name, title)
{
  TArtCore::Info(__FILE__,"Creating NEBULA setup...");

  pmap = new TArtNEBULAPlaParaMap;

  fStoreManager = TArtStoreManager::Instance();
  fStoreManager->AddParameters(this);
}

//__________________________________________________________
TArtNEBULAParameters::~TArtNEBULAParameters() 
{
  TArtNEBULAPlaParaMap::iterator itr = pmap->begin();
  std::vector<const TArtNEBULAPlaPara*> pvec;
  while(itr != pmap->end()){
    std::vector<const TArtNEBULAPlaPara*>::iterator it =
      find(pvec.begin(), pvec.end(), itr->second);
    if(it == pvec.end()){
      pvec.push_back(itr->second);
      delete itr->second;
    }
    itr->second = 0;
    ++itr;
  }
}

//__________________________________________________________
Bool_t TArtNEBULAParameters::LoadParameter(const char *xmlfile)
{
  TArtCore::Info(__FILE__,"Load parameter from %s", xmlfile);
  TDOMParser* domParser = new TDOMParser();
  domParser->SetValidate(false);
  Int_t parsecode = domParser->ParseFile(xmlfile);
  if(parsecode < 0){
    cerr << domParser->GetParseCodeMessage(parsecode) << endl;
    return false;
  }
  TXMLNode* node = domParser->GetXMLDocument()->GetRootNode();
  ParseParaList(node->GetChildren());
  return true;
}

//__________________________________________________________
void TArtNEBULAParameters::ParseParaList(TXMLNode *node)
{
  for(; node; node = node->GetNextNode()){
    if(node->GetNodeType() == TXMLNode::kXMLElementNode){ // Element Node
      if(strcmp(node->GetNodeName(), "NEBULA") == 0){
	ParseNEBULAPlaPara(node->GetChildren());
      }
    }
  }
  return;
}

//__________________________________________________________
TArtNEBULAPlaPara *TArtNEBULAParameters::ParseNEBULAPlaPara(TXMLNode *node)
{
  Int_t     serialnumber = -1;
  Int_t     id = -1;
  TString   udlr;
  TString   name;
  Int_t     fpl = -1;
  Int_t     layer = -1;
  Int_t     sublayer = -1;
  Double_t  pos[3] = {0, 0, 0};
  Double_t  tcal = 0;
  Double_t  toff = 0;
  Double_t  qcal = 0;
  Double_t  qped = 0;
  Int_t     ignore = 0;
  Int_t tdc_geo, tdc_ch, adc_geo, adc_ch;

  for(; node; node = node->GetNextNode()){
    if(node->GetNodeType() == TXMLNode::kXMLElementNode){ // Element Node
      if(strcmp(node->GetNodeName(), "SerialNumber") == 0) 
	serialnumber = atoi(node->GetText());
      if(strcmp(node->GetNodeName(), "ID") == 0) 
	id = atoi(node->GetText());
      if(strcmp(node->GetNodeName(), "UDLR") == 0)
	udlr = node->GetText();
      if(strcmp(node->GetNodeName(), "Name") == 0) 
	name = node->GetText();
      if(strcmp(node->GetNodeName(), "FPl") == 0) 
	fpl = atoi(node->GetText());
      if(strcmp(node->GetNodeName(), "Layer") == 0) 
	layer = atoi(node->GetText());
      if(strcmp(node->GetNodeName(), "SubLayer") == 0) 
	sublayer = atoi(node->GetText());
      if(strcmp(node->GetNodeName(), "PosX") == 0) 
	pos[0] = atof(node->GetText());
      if(strcmp(node->GetNodeName(), "PosY") == 0) 
	pos[1] = atof(node->GetText());
      if(strcmp(node->GetNodeName(), "PosZ") == 0) 
	pos[2] = atof(node->GetText());
      if(strcmp(node->GetNodeName(), "TCal") == 0)
	tcal = atof(node->GetText());
      if(strcmp(node->GetNodeName(), "TOff") == 0)
	toff = atof(node->GetText());
      if(strcmp(node->GetNodeName(), "QCal") == 0)
	qcal = atof(node->GetText());
      if(strcmp(node->GetNodeName(), "QPed") == 0)
	qped = atof(node->GetText());
      if(strcmp(node->GetNodeName(), "Ignore") == 0)
	ignore = atoi(node->GetText());

      if(strcmp(node->GetNodeName(), "tdc_geo") == 0) 
	tdc_geo = atoi(node->GetText());
      if(strcmp(node->GetNodeName(), "tdc_ch") == 0) 
	tdc_ch = atoi(node->GetText());
      if(strcmp(node->GetNodeName(), "adc_geo") == 0) 
	adc_geo = atoi(node->GetText());
      if(strcmp(node->GetNodeName(), "adc_ch") == 0) 
	adc_ch = atoi(node->GetText());
    }
  }
  
  //  TArtCore::Info(__FILE__,"Reading database for %s NEBULA",name.Data());
  TArtNEBULAPlaPara* para =
    new TArtNEBULAPlaPara(serialnumber, id, udlr, name, fpl,
			  layer, sublayer, pos,
			  tcal, toff, qcal, qped, ignore);
  para->SetMap(tdc_geo, tdc_ch, adc_geo, adc_ch);

  pmap->insert(TArtNEBULAPlaParaPair(para->GetTDCMap(), para));
  pmap->insert(TArtNEBULAPlaParaPair(para->GetADCMap(), para));

  return para;
}

const TArtNEBULAPlaPara* TArtNEBULAParameters::FindNEBULAPlaPara(const TArtRIDFMap &rmap) const
{
  TArtNEBULAPlaParaMap::const_iterator itr = pmap->find(rmap);
  if(itr != pmap->end()) return itr->second;
  else return 0;
}
